var fs = require("fs"), path = require("path"), s = require("string");


exports.generate = function(options, callback) {		
	fs.readFile(options.input, function(err, jsonResults) {
		if (err) {callback(err);} else {
			callback(null, buildStructure(JSON.parse(jsonResults.toString())));
		}
	});
	
	function buildStructure(jsonReport) {
		var structure = {
			name : "Acceptance Tests",
			uri : options.features,
			features : [],
			children : []
		};
		
		jsonReport.features.forEach(function(feature) {		
			addToStructure(structure, feature);
		});
		
		buildStructureStatus(structure);
		return structure;
	}
	
	function addToStructure(structure, feature) {
		var pathParts = normalizeFeaturePath(feature.uri).split(path.sep);
		var currentLevel = structure;
		for(var i=0; i<pathParts.length - 1; i++) {
			var pathPart = pathParts[i];
			currentLevel = ensurePath(currentLevel.children, pathPart, pathParts.slice(0, i+1).join(path.sep));
		}
		currentLevel.features.push(feature);
	}
	
	function ensurePath(childStructure, pathPart, parentPath) {
		var found = null;
		var friendlyName = s(pathPart.replace(/_/g, " ")).capitalize().toString();
		for(var i=0; i<childStructure.length; i++) {
			if (childStructure[i].pathPart === pathPart) {
				found = childStructure[i];
				break;
			}
		}
		if (!found) {
			found = {
				pathPart : pathPart,
				name : friendlyName,
				uri : path.join(options.features, parentPath),
				features : [],
				children : []
			};
			childStructure.push(found);
		}
		return found;
	}
	
	function normalizeFeaturePath(filePath) {	
		return path.relative(options.features, filePath);
	}

	function buildStructureStatus(structure) {
		var status = StatusManager();
		structure.features.forEach(function(feature) {
			feature.status="success";
			if (feature.hasFailed) {
				feature.status = "fail";
				status.update("fail");
			} else if (feature.hasUndefined) {
				feature.status = "undefined";
				status.update("undefined");
			} else if (feature.hasWip) {
				feature.status = "wip";
				status.update("wip");
			} 
		});
				
		structure.children.forEach(function(child) {
			buildStructureStatus(child);
			status.update(child.status);
		});		
		
		structure.status = status.get();
	}
};

function StatusManager() {
	var order = ["success", "wip", "undefined", "fail"];
	var statusIndex = 0;
	return {
		update : function(newState) {
			if (order.indexOf(newState) > statusIndex) {
				statusIndex = order.indexOf(newState);
			}
		},
		get : function() { return order[statusIndex]; }
	};
}